###################################################
######## TP5 - Solutions #########
###################################################

rm(list=ls())
graphics.off()

library(DiceKriging)
help(package="DiceKriging")
?km

# test function 
myfunc <- function(x){
  return( sin(30*(x - 0.9)^4)*cos(2*(x - 0.9)) + (x - 0.9)/2)
}

# design of experiments
nx <- 5
x <- seq(0,1,le=nx)
y <- myfunc(x)

# vizualisation
npred <- 100
xpred <- seq(0,1,le=npred)
ypred_true <- myfunc(xpred)

x11()
plot(xpred,ypred_true,type="l",xlab="x",ylab="y")
points(x,y)

# parameters of kriging
mu = 0
sig2 = 0.5
theta = 0.2

# Building the kriging
krig <- km(~1,design=data.frame(x=x), response=y, covtype="gauss", 
               coef.var=sig2, coef.trend=mu, coef.cov=theta)

# predictions of kriging
ypred <- predict(object=krig, newdata=xpred, type="SK", checkNames=F)

x11()
plot(xpred,myfunc(xpred),type="l",xlab="x",ylab="y",lwd=2,
     main="kriging predictions",ylim=c(min(ypred$lower95),max(ypred$upper95)))
points(x,y,col=2,pch=2,lwd=2)
lines(xpred,ypred$mean,col=4,lwd=2)
lines(xpred,ypred$lower95,col=4,lty=2)
lines(xpred,ypred$upper95,col=4,lty=2)

#########################
# Conditional simulations

q1 = quantile(ypred_true,0.95)
print(c("True 0.95-quantile = ",q1))
abline(h=q1)

q2 = quantile(ypred$mean,0.95)
print(c("Kriging 0.95-quantile = ",q2))
abline(h=q2,col=4)

nsim = 1e4
zsim = simulate(krig,nsim=nsim,cond=TRUE,newdata=data.frame(x=xpred),nugget.sim=1e-5)
q=rep(0,nsim)
for (i in 1:nsim){
  q[i] = quantile(zsim[i,],0.95)
}
for (i in 1:20) lines(xpred,zsim[i,],col=2)

q3 = mean(q)
print(c("Quantile by conditional simul. =",q3))
abline(h=q3,col=2)

res <- quantile(q,c(0.05,0.95))
print(c("0.95-quantile CI =",res))

# Histogram of the output
x11()
hist(ypred_true)
abline(v=q1)
abline(v=q2,col=4)
abline(v=q3,col=2)

